package aerogen;

import org.bukkit.Material;

public class Schematic {

	protected static final byte caveSpace = (byte) Material.ENDER_STONE.getId();
	
	public static int maxSchematicChunkSize = 16;
	
	/** length of the schematic, x-ways in MineCraft **/
	protected int length = 0;
	/** bredth of the schematic, z-ways in MineCraft **/
	protected int bredth = 0;
	/** height of the schematic, y-ways in MineCraft **/
	protected int height = 0;
	private int x, y, z;
	private int chunkXMin, chunkXMax, chunkZMin, chunkZMax;
	private boolean[][] chunkComplete;
	protected byte[] blocks;
	
	public Schematic(int x, int y, int z, int length, int height, int bredth) {
		super();
		this.length = length;
		this.bredth = bredth;
		this.height = height;
		this.x = x;
		this.y = y;
		this.z = z;
		this.blocks = new byte[length*bredth*height];
		this.chunkXMin = floor(x / 16.0)-1;
		this.chunkZMin = floor(z / 16.0)-1;
		this.chunkXMax = floor((x+length) / 16.0);
		this.chunkZMax = floor((z+bredth) / 16.0);
		this.chunkComplete = new boolean[chunkXMax-chunkXMin+1][chunkZMax-chunkZMin+1];
//		AeroGen.log("New Schematic ["+chunkXMin+" - "+chunkXMax+", "+chunkZMin+" - "+chunkZMax+"]");
	}

	public void setBlock(int x, int y, int z, byte material) {
		blocks[(x*bredth + z)*height + y] = material;
	}
	
	public byte getBlock(int x, int y, int z) {
		return blocks[(x*bredth + z)*height + y];
	}

	public void setBlockSafe(int x, int y, int z, byte material) {
		if(x >= 0 && x < length && z >= 0 && z < bredth && y >= 0 && y < height) {
			blocks[(x*bredth + z)*height + y] = material;
		}
	}
	
	public byte getBlockSafe(int x, int y, int z) {
		if(x >= 0 && x < length && z >= 0 && z < bredth && y >=0 && y < height) {
			return blocks[(x*bredth + z)*height + y];
		}
		return 0;
	}
	
	public int getLength() {
		return length;
	}

	public int getBredth() {
		return bredth;
	}

	public int getHeight() {
		return height;
	}

	public byte[] getBlocks() {
		return blocks;
	}
	
	public int floor(double d) {
		if(d >= 0) {
			return (int) d;
		} else {
			return (int) d - 1;
		}
	}
	
	public int ceil(double d) {
		if(d >= 0) {
			return (int) d + 1;
		} else {
			return (int) d;
		}
	}
	
	public boolean addToChunk(byte[] chunk, int chunkX, int chunkZ) {
		if(chunkX >= chunkXMin && chunkX <= chunkXMax && chunkZ >= chunkZMin && chunkZ <= chunkZMax) { 
			
			int minX = this.x-chunkX*16;
			if(minX < 0) minX = 0;
			
			int minZ = this.z-chunkZ*16;
			if(minZ < 0) minZ = 0;
			
			int maxX = this.x+this.length-chunkX*16;
			if(maxX > 16) maxX = 16;
			
			int maxZ = this.z+this.bredth-chunkZ*16;
			if(maxZ > 16) maxZ = 16;
			
			int initX = chunkX*16-this.x;
			int initZ = chunkZ*16-this.z;
			
			for(int i = minX; i < maxX; i++) {
				for(int k = minZ; k < maxZ; k++) {
					
					int x = initX+i;
					int z = initZ+k;
					
					int maxY = 128-this.y;
					if(maxY > height) maxY = height;
					int minY = 0 - this.y;
					if(minY < 0) minY = 0;
					
					int blockIndex = (x*bredth + z)*height;
					int chunkIndex = (i*16 + k)*128 + this.y;
					
					for(int j = minY; j < maxY; j++) {
						if(blocks[blockIndex+j] != 0) {
							if(chunk[chunkIndex+j] != caveSpace) {
								chunk[chunkIndex+j] = blocks[blockIndex+j];
							}
						}
					}
				}
			}
			
			this.chunkComplete[chunkX-chunkXMin][chunkZ-chunkZMin] = true;
			
		}
	
		for(int i = 0; i <= chunkXMax-chunkXMin; i++) {
			for(int k = 0; k <= chunkZMax-chunkZMin; k++) {
				if(!this.chunkComplete[i][k]) return false;
			}
		}
		return true;
		
	}
	
	public void completeChunk(int chunkX, int chunkZ) {
		if(chunkX >= chunkXMin && chunkX <= chunkXMax && chunkZ >= chunkZMin && chunkZ <= chunkZMax) { 
			this.chunkComplete[chunkX-chunkXMin][chunkZ-chunkZMin] = true;
		}
	}
	
}
